@@ -1,8 +1,11 @@
-import os
 import logging
+import os
+from typing import Any, Dict, Optional, Union
 
 from twisted.python.failure import Failure
 
+from scrapy import Request, Spider
+from scrapy.http import Response
 from scrapy.utils.request import referer_str
 
 SCRAPEDMSG = "Scraped from %(src)s" + os.linesep + "%(item)s"
@@ -52,94 +55,109 @@ class LogFormatter:
                     }
     """
 
-    def crawled(self, request, response, spider):
+    def crawled(self, request: Request, response: Response, spider: Spider) -> dict:
         """Logs a message when the crawler finds a webpage."""
-        request_flags = f' {str(request.flags)}' if request.flags else ''
-        response_flags = f' {str(response.flags)}' if response.flags else ''
+        request_flags = f" {str(request.flags)}" if request.flags else ""
+        response_flags = f" {str(response.flags)}" if response.flags else ""
         return {
-            'level': logging.DEBUG,
-            'msg': CRAWLEDMSG,
-            'args': {
-                'status': response.status,
-                'request': request,
-                'request_flags': request_flags,
-                'referer': referer_str(request),
-                'response_flags': response_flags,
+            "level": logging.DEBUG,
+            "msg": CRAWLEDMSG,
+            "args": {
+                "status": response.status,
+                "request": request,
+                "request_flags": request_flags,
+                "referer": referer_str(request),
+                "response_flags": response_flags,
                 # backward compatibility with Scrapy logformatter below 1.4 version
-                'flags': response_flags
-            }
+                "flags": response_flags,
+            },
         }
 
-    def scraped(self, item, response, spider):
+    def scraped(
+        self, item: Any, response: Union[Response, Failure], spider: Spider
+    ) -> dict:
         """Logs a message when an item is scraped by a spider."""
+        src: Any
         if isinstance(response, Failure):
             src = response.getErrorMessage()
         else:
             src = response
         return {
-            'level': logging.DEBUG,
-            'msg': SCRAPEDMSG,
-            'args': {
-                'src': src,
-                'item': item,
-            }
+            "level": logging.DEBUG,
+            "msg": SCRAPEDMSG,
+            "args": {
+                "src": src,
+                "item": item,
+            },
         }
 
-    def dropped(self, item, exception, response, spider):
+    def dropped(
+        self, item: Any, exception: BaseException, response: Response, spider: Spider
+    ) -> dict:
         """Logs a message when an item is dropped while it is passing through the item pipeline."""
         return {
-            'level': logging.WARNING,
-            'msg': DROPPEDMSG,
-            'args': {
-                'exception': exception,
-                'item': item,
-            }
+            "level": logging.WARNING,
+            "msg": DROPPEDMSG,
+            "args": {
+                "exception": exception,
+                "item": item,
+            },
         }
 
-    def item_error(self, item, exception, response, spider):
+    def item_error(
+        self, item: Any, exception, response: Response, spider: Spider
+    ) -> dict:
         """Logs a message when an item causes an error while it is passing
         through the item pipeline.
 
         .. versionadded:: 2.0
         """
         return {
-            'level': logging.ERROR,
-            'msg': ITEMERRORMSG,
-            'args': {
-                'item': item,
-            }
+            "level": logging.ERROR,
+            "msg": ITEMERRORMSG,
+            "args": {
+                "item": item,
+            },
         }
 
-    def spider_error(self, failure, request, response, spider):
+    def spider_error(
+        self, failure: Failure, request: Request, response: Response, spider: Spider
+    ) -> dict:
         """Logs an error message from a spider.
 
         .. versionadded:: 2.0
         """
         return {
-            'level': logging.ERROR,
-            'msg': SPIDERERRORMSG,
-            'args': {
-                'request': request,
-                'referer': referer_str(request),
-            }
+            "level": logging.ERROR,
+            "msg": SPIDERERRORMSG,
+            "args": {
+                "request": request,
+                "referer": referer_str(request),
+            },
         }
 
-    def download_error(self, failure, request, spider, errmsg=None):
+    def download_error(
+        self,
+        failure: Failure,
+        request: Request,
+        spider: Spider,
+        errmsg: Optional[str] = None,
+    ) -> dict:
         """Logs a download error message from a spider (typically coming from
         the engine).
 
         .. versionadded:: 2.0
         """
-        args = {'request': request}
+        args: Dict[str, Any] = {"request": request}
         if errmsg:
             msg = DOWNLOADERRORMSG_LONG
-            args['errmsg'] = errmsg
+            args["errmsg"] = errmsg
         else:
             msg = DOWNLOADERRORMSG_SHORT
         return {
-            'level': logging.ERROR,
-            'msg': msg,
-            'args': args,
+            "level": logging.ERROR,
+            "msg": msg,
+            "args": args,
         }
 
     @classmethod
