@@ -7,18 +7,22 @@ from warnings import warn
 from twisted.internet.defer import Deferred, DeferredList
 from twisted.python.failure import Failure
 
+from scrapy.http.request import NO_CALLBACK
 from scrapy.settings import Settings
 from scrapy.utils.datatypes import SequenceExclude
-from scrapy.utils.defer import mustbe_deferred, defer_result
+from scrapy.utils.defer import defer_result, mustbe_deferred
 from scrapy.utils.deprecate import ScrapyDeprecationWarning
-from scrapy.utils.misc import arg_to_iter
 from scrapy.utils.log import failure_to_exc_info
+from scrapy.utils.misc import arg_to_iter
 
 logger = logging.getLogger(__name__)
 
 
-class MediaPipeline:
+def _DUMMY_CALLBACK(response):
+    return response
 
+
+class MediaPipeline:
     LOG_FAILED_RESULTS = True
 
     class SpiderInfo:
@@ -34,12 +38,10 @@ class MediaPipeline:
 
         if isinstance(settings, dict) or settings is None:
             settings = Settings(settings)
-        resolve = functools.partial(self._key_for_pipe,
-                                    base_class_name="MediaPipeline",
-                                    settings=settings)
-        self.allow_redirects = settings.getbool(
-            resolve('MEDIA_ALLOW_REDIRECTS'), False
+        resolve = functools.partial(
+            self._key_for_pipe, base_class_name="MediaPipeline", settings=settings
         )
+        self.allow_redirects = settings.getbool(resolve("MEDIA_ALLOW_REDIRECTS"), False)
         self._handle_statuses(self.allow_redirects)
 
         # Check if deprecated methods are being used and make them compatible
@@ -64,7 +66,8 @@ class MediaPipeline:
         if (
             not base_class_name
             or class_name == base_class_name
-            or settings and not settings.get(formatted_key)
+            or settings
+            and not settings.get(formatted_key)
         ):
             return key
         return formatted_key
@@ -91,9 +94,12 @@ class MediaPipeline:
 
     def _process_request(self, request, info, item):
         fp = self._fingerprinter.fingerprint(request)
-        cb = request.callback or (lambda _: _)
+        if not request.callback or request.callback is NO_CALLBACK:
+            cb = _DUMMY_CALLBACK
+        else:
+            cb = request.callback
         eb = request.errback
-        request.callback = None
+        request.callback = NO_CALLBACK
         request.errback = None
 
         # Return cached result if request was already seen
@@ -113,16 +119,23 @@ class MediaPipeline:
         dfd = mustbe_deferred(self.media_to_download, request, info, item=item)
         dfd.addCallback(self._check_media_to_download, request, info, item=item)
         dfd.addBoth(self._cache_result_and_execute_waiters, fp, info)
-        dfd.addErrback(lambda f: logger.error(
-            f.value, exc_info=failure_to_exc_info(f), extra={'spider': info.spider})
+        dfd.addErrback(
+            lambda f: logger.error(
+                f.value, exc_info=failure_to_exc_info(f), extra={"spider": info.spider}
+            )
         )
         return dfd.addBoth(lambda _: wad)  # it must return wad at last
 
     def _make_compatible(self):
         """Make overridable methods of MediaPipeline and subclasses backwards compatible"""
         methods = [
-            "file_path", "thumb_path", "media_to_download", "media_downloaded",
-            "file_downloaded", "image_downloaded", "get_images"
+            "file_path",
+            "thumb_path",
+            "media_to_download",
+            "media_downloaded",
+            "file_downloaded",
+            "image_downloaded",
+            "get_images",
         ]
 
         for method_name in methods:
@@ -139,7 +152,7 @@ class MediaPipeline:
             if self._expects_item[func.__name__]:
                 return func(*args, **kwargs)
 
-            kwargs.pop('item', None)
+            kwargs.pop("item", None)
             return func(*args, **kwargs)
 
         return wrapper
@@ -148,19 +161,22 @@ class MediaPipeline:
         sig = signature(func)
         self._expects_item[func.__name__] = True
 
-        if 'item' not in sig.parameters:
+        if "item" not in sig.parameters:
             old_params = str(sig)[1:-1]
             new_params = old_params + ", *, item=None"
-            warn(f'{func.__name__}(self, {old_params}) is deprecated, '
-                 f'please use {func.__name__}(self, {new_params})',
-                 ScrapyDeprecationWarning, stacklevel=2)
+            warn(
+                f"{func.__name__}(self, {old_params}) is deprecated, "
+                f"please use {func.__name__}(self, {new_params})",
+                ScrapyDeprecationWarning,
+                stacklevel=2,
+            )
             self._expects_item[func.__name__] = False
 
     def _modify_media_request(self, request):
         if self.handle_httpstatus_list:
-            request.meta['handle_httpstatus_list'] = self.handle_httpstatus_list
+            request.meta["handle_httpstatus_list"] = self.handle_httpstatus_list
         else:
-            request.meta['handle_httpstatus_all'] = True
+            request.meta["handle_httpstatus_all"] = True
 
     def _check_media_to_download(self, result, request, info, item):
         if result is not None:
@@ -169,14 +185,22 @@ class MediaPipeline:
             # this ugly code was left only to support tests. TODO: remove
             dfd = mustbe_deferred(self.download_func, request, info.spider)
             dfd.addCallbacks(
-                callback=self.media_downloaded, callbackArgs=(request, info), callbackKeywords={'item': item},
-                errback=self.media_failed, errbackArgs=(request, info))
+                callback=self.media_downloaded,
+                callbackArgs=(request, info),
+                callbackKeywords={"item": item},
+                errback=self.media_failed,
+                errbackArgs=(request, info),
+            )
         else:
             self._modify_media_request(request)
             dfd = self.crawler.engine.download(request)
             dfd.addCallbacks(
-                callback=self.media_downloaded, callbackArgs=(request, info), callbackKeywords={'item': item},
-                errback=self.media_failed, errbackArgs=(request, info))
+                callback=self.media_downloaded,
+                callbackArgs=(request, info),
+                callbackKeywords={"item": item},
+                errback=self.media_failed,
+                errbackArgs=(request, info),
+            )
         return dfd
 
     def _cache_result_and_execute_waiters(self, result, fp, info):
@@ -207,9 +231,9 @@ class MediaPipeline:
             #
             # This problem does not occur in Python 2.7 since we don't have
             # Exception Chaining (https://www.python.org/dev/peps/pep-3134/).
-            context = getattr(result.value, '__context__', None)
+            context = getattr(result.value, "__context__", None)
             if isinstance(context, StopIteration):
-                setattr(result.value, '__context__', None)
+                setattr(result.value, "__context__", None)
 
         info.downloading.remove(fp)
         info.downloaded[fp] = result  # cache result
@@ -239,10 +263,10 @@ class MediaPipeline:
             for ok, value in results:
                 if not ok:
                     logger.error(
-                        '%(class)s found errors processing %(item)s',
-                        {'class': self.__class__.__name__, 'item': item},
+                        "%(class)s found errors processing %(item)s",
+                        {"class": self.__class__.__name__, "item": item},
                         exc_info=failure_to_exc_info(value),
-                        extra={'spider': info.spider}
+                        extra={"spider": info.spider},
                     )
         return item
 
