@@ -3,7 +3,6 @@ import logging
 
 from scrapy.utils.misc import create_instance
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -18,11 +17,11 @@ def _path_safe(text):
     >>> _path_safe('some@symbol?').startswith('some_symbol_')
     True
     """
-    pathable_slot = "".join([c if c.isalnum() or c in '-._' else '_' for c in text])
+    pathable_slot = "".join([c if c.isalnum() or c in "-._" else "_" for c in text])
     # as we replace some letters we can get collision for different slots
     # add we add unique part
-    unique_slot = hashlib.md5(text.encode('utf8')).hexdigest()
-    return '-'.join([pathable_slot, unique_slot])
+    unique_slot = hashlib.md5(text.encode("utf8")).hexdigest()
+    return "-".join([pathable_slot, unique_slot])
 
 
 class ScrapyPriorityQueue:
@@ -77,7 +76,7 @@ class ScrapyPriorityQueue:
             self.downstream_queue_cls,
             None,
             self.crawler,
-            self.key + '/' + str(key),
+            self.key + "/" + str(key),
         )
 
     def priority(self, request):
@@ -128,7 +127,6 @@ class ScrapyPriorityQueue:
 
 
 class DownloaderInterface:
-
     def __init__(self, crawler):
         self.downloader = crawler.engine.downloader
 
@@ -139,14 +137,14 @@ class DownloaderInterface:
         return self.downloader._get_slot_key(request, None)
 
     def _active_downloads(self, slot):
-        """ Return a number of requests in a Downloader for a given slot """
+        """Return a number of requests in a Downloader for a given slot"""
         if slot not in self.downloader.slots:
             return 0
         return len(self.downloader.slots[slot].active)
 
 
 class DownloaderAwarePriorityQueue:
-    """ PriorityQueue which takes Downloader activity into account:
+    """PriorityQueue which takes Downloader activity into account:
     domains (slots) with the least amount of active downloads are dequeued
     first.
     """
@@ -156,17 +154,21 @@ class DownloaderAwarePriorityQueue:
         return cls(crawler, downstream_queue_cls, key, startprios)
 
     def __init__(self, crawler, downstream_queue_cls, key, slot_startprios=()):
-        if crawler.settings.getint('CONCURRENT_REQUESTS_PER_IP') != 0:
-            raise ValueError(f'"{self.__class__}" does not support CONCURRENT_REQUESTS_PER_IP')
+        if crawler.settings.getint("CONCURRENT_REQUESTS_PER_IP") != 0:
+            raise ValueError(
+                f'"{self.__class__}" does not support CONCURRENT_REQUESTS_PER_IP'
+            )
 
         if slot_startprios and not isinstance(slot_startprios, dict):
-            raise ValueError("DownloaderAwarePriorityQueue accepts "
-                             "``slot_startprios`` as a dict; "
-                             f"{slot_startprios.__class__!r} instance "
-                             "is passed. Most likely, it means the state is"
-                             "created by an incompatible priority queue. "
-                             "Only a crawl started with the same priority "
-                             "queue class can be resumed.")
+            raise ValueError(
+                "DownloaderAwarePriorityQueue accepts "
+                "``slot_startprios`` as a dict; "
+                f"{slot_startprios.__class__!r} instance "
+                "is passed. Most likely, it means the state is"
+                "created by an incompatible priority queue. "
+                "Only a crawl started with the same priority "
+                "queue class can be resumed."
+            )
 
         self._downloader_interface = DownloaderInterface(crawler)
         self.downstream_queue_cls = downstream_queue_cls
@@ -181,7 +183,7 @@ class DownloaderAwarePriorityQueue:
         return ScrapyPriorityQueue(
             self.crawler,
             self.downstream_queue_cls,
-            self.key + '/' + _path_safe(slot),
+            self.key + "/" + _path_safe(slot),
             startprios,
         )
 
