@@ -1,39 +1,37 @@
-import json
 import datetime
 import decimal
+import json
 
-from itemadapter import is_item, ItemAdapter
+from itemadapter import ItemAdapter, is_item
 from twisted.internet import defer
 
 from scrapy.http import Request, Response
 
 
 class ScrapyJSONEncoder(json.JSONEncoder):
-
     DATE_FORMAT = "%Y-%m-%d"
     TIME_FORMAT = "%H:%M:%S"
 
     def default(self, o):
         if isinstance(o, set):
             return list(o)
-        elif isinstance(o, datetime.datetime):
+        if isinstance(o, datetime.datetime):
             return o.strftime(f"{self.DATE_FORMAT} {self.TIME_FORMAT}")
-        elif isinstance(o, datetime.date):
+        if isinstance(o, datetime.date):
             return o.strftime(self.DATE_FORMAT)
-        elif isinstance(o, datetime.time):
+        if isinstance(o, datetime.time):
             return o.strftime(self.TIME_FORMAT)
-        elif isinstance(o, decimal.Decimal):
+        if isinstance(o, decimal.Decimal):
             return str(o)
-        elif isinstance(o, defer.Deferred):
+        if isinstance(o, defer.Deferred):
             return str(o)
-        elif is_item(o):
+        if is_item(o):
             return ItemAdapter(o).asdict()
-        elif isinstance(o, Request):
+        if isinstance(o, Request):
             return f"<{type(o).__name__} {o.method} {o.url}>"
-        elif isinstance(o, Response):
+        if isinstance(o, Response):
             return f"<{type(o).__name__} {o.status} {o.url}>"
-        else:
-            return super().default(o)
+        return super().default(o)
 
 
 class ScrapyJSONDecoder(json.JSONDecoder):
