@@ -1,24 +1,28 @@
+from typing import Any, Optional, cast
+
 import OpenSSL._util as pyOpenSSLutil
 import OpenSSL.SSL
+import OpenSSL.version
+from OpenSSL.crypto import X509Name
 
 from scrapy.utils.python import to_unicode
 
 
-def ffi_buf_to_string(buf):
+def ffi_buf_to_string(buf: Any) -> str:
     return to_unicode(pyOpenSSLutil.ffi.string(buf))
 
 
-def x509name_to_string(x509name):
+def x509name_to_string(x509name: X509Name) -> str:
     # from OpenSSL.crypto.X509Name.__repr__
-    result_buffer = pyOpenSSLutil.ffi.new("char[]", 512)
+    result_buffer: Any = pyOpenSSLutil.ffi.new("char[]", 512)
     pyOpenSSLutil.lib.X509_NAME_oneline(
-        x509name._name, result_buffer, len(result_buffer)
+        x509name._name, result_buffer, len(result_buffer)  # type: ignore[attr-defined]
     )
 
     return ffi_buf_to_string(result_buffer)
 
 
-def get_temp_key_info(ssl_object):
+def get_temp_key_info(ssl_object: Any) -> Optional[str]:
     # adapted from OpenSSL apps/s_cb.c::ssl_print_tmp_key()
     if not hasattr(pyOpenSSLutil.lib, "SSL_get_server_tmp_key"):
         # removed in cryptography 40.0.0
@@ -53,8 +57,10 @@ def get_temp_key_info(ssl_object):
     return ", ".join(key_info)
 
 
-def get_openssl_version():
-    system_openssl = OpenSSL.SSL.SSLeay_version(OpenSSL.SSL.SSLEAY_VERSION).decode(
-        "ascii", errors="replace"
+def get_openssl_version() -> str:
+    # https://github.com/python/typeshed/issues/10024
+    system_openssl_bytes = cast(
+        bytes, OpenSSL.SSL.SSLeay_version(OpenSSL.SSL.SSLEAY_VERSION)
     )
+    system_openssl = system_openssl_bytes.decode("ascii", errors="replace")
     return f"{OpenSSL.version.__version__} ({system_openssl})"
